/*
	La version un peu plus dveloppe de l'applet "maison"
	qui dessine maintenant des ovales, des rectangles
	et des lignes...
	Utilise maintenant un design pattern factory pour
	construire des objets...
	C'est toujours trs joli pour les enfants...

	(C) J. Ferber - Juillet 2000
*/
package maison;

import java.awt.*;
import java.applet.Applet;
import java.awt.event.*;
import java.util.Vector;
import javax.swing.*;
import java.io.*;

/** La classe DrawingPane qui implmente toute la partie graphique
 *  sous la forme d'un composant...
*/
public class DrawingPane extends JComponent
						implements MouseListener, MouseMotionListener
{
        static final int MODE_SELECT = 0;
        static final int MODE_DEFINE = 1;

        static final int GRIDSIZE=20;

        int mode=MODE_DEFINE;
        boolean grid=true;
        Figure figSelected=null;

        int newX, newY;
        int oldX, oldY;
        int prevX, prevY;

	int couleur = 0; // la couleur initiale
 	public void incrColor(){
			couleur=(couleur+1) % 12;
  	}

	public void setColor(int c){
	  couleur = c;
	}


	Figure fig; // la figure courante..
	Vector figures=new Vector(); // la collection des figures  afficher

	void unselect(boolean redisp){
         if (figSelected != null) {
			figSelected.setSelected(false);
			figSelected = null;
		 }
		if (redisp)
			repaint();
	}

 	public void setMode(int c){mode = c;}
  	public void toggleGrid(){
       	grid = !grid;
        repaint();
    }

	DrawingPane() {
		this.addMouseListener(this);
  		this.addMouseMotionListener(this);

		fig = new Rectangle();
	}

 	/** recherche l'lment dans lequel on a clicke */
	Figure find(int x, int y)
	{	int n = figures.size();
		Point p = new Point(x,y);
		Figure c;

		for(int i=0;i<n;i++)
        {   c=(Figure) figures.elementAt(i);
            if(c.contains(p))
                return c;
        }
        return null;
    }

	// la gestion des vnements

	public void mousePressed(MouseEvent e) {

       if (mode == MODE_DEFINE){
	   		unselect(false);
	   		if (fig != null){
           		oldX = e.getX();
           		oldY = e.getY();
         		prevX = oldX;
          		prevY = oldY;
           }
        } else if (mode == MODE_SELECT){
			Figure oldSelect = figSelected;
            figSelected = find(e.getX(),e.getY());
            prevX = e.getX();
            prevY = e.getY();
			if (figSelected != oldSelect){
				if (oldSelect != null)
					oldSelect.setSelected(false);
				if (figSelected != null){
					figSelected.setSelected(true);
				}
			} else figSelected.doReSelect(e.getX(),e.getY());
        }
	}

	public void mouseDragged(MouseEvent e) {
     	int x = e.getX();
      	int y = e.getY();

       	Graphics g=getGraphics();

		if (mode == MODE_DEFINE){
      		//System.out.println("draw: " + oldX + ", " + oldY + ", "
        	//					+ (x-oldX) + ", " + (y-oldY));
	       	g.setXORMode(Color.darkGray);
         	if (fig != null){
              	fig.drawBuild(g, oldX, oldY, prevX, prevY);
          	}
         	prevX = x;
          	prevY = y;
         	if (fig != null){
              	fig.drawBuild(g, oldX, oldY, x, y);
          	}
	       	g.setPaintMode();
		} else if ((mode == MODE_SELECT) && (figSelected != null)){
      		g.setXORMode(Color.darkGray);
         	figSelected.doDrag(g, x-prevX, y-prevY);
          	prevX = x;
            	prevY = y;
	       	g.setPaintMode();
  		}

 	}


	public void mouseReleased(MouseEvent e) {
 		Figure f;
       		if (mode == MODE_DEFINE){
			if (fig != null){
	           newX = e.getX();
	           newY = e.getY();
			   f = fig.cloneFig(oldX,oldY,newX,newY,couleur);
	           if (f != null){
		          f.dessiner(getGraphics());
		          figures.addElement(f);
	           }
			}
		} else if (mode == MODE_SELECT){
      			repaint();
        }
    }

    /** la methode paint qui affiche l'ensemble des figures
     */
	public void paint(Graphics g){
     	setBackground(Color.white);
		g.setColor(Color.white);
      	g.fillRect(0,0,getSize().width,getSize().height);
      	g.setColor(Color.black);
      	g.drawRect(1,1,getSize().width-2,getSize().height-2);
        if (grid) paintGrid(g);
		for(int i=0;i<figures.size();i++)
			((Figure)figures.elementAt(i)).dessiner(g);
	}

	/** la methode qui active l'animation des figures
	*/
	public synchronized void  runFigures(){
    	for (int i=0; i < figures.size(); i++){
    		Figure e = (Figure)figures.elementAt(i);
			if (e != null)
				e.activate(this);
		}
		repaint();
    }

  	/** la methode de dessin d'une grille
    */
    void paintGrid(Graphics g){
		Dimension d = getSize();
  		int height = d.height;
    	int width = d.width;
     	int nh = height/GRIDSIZE;
     	int nw = width/GRIDSIZE;

     	g.setColor(Color.lightGray);
     	for (int i=1;i <= nh;i++){
          	g.drawLine(0,i*GRIDSIZE,width,i*GRIDSIZE);
      	}
        for (int j=1;j <= nw;j++){
          	g.drawLine(j*GRIDSIZE,0,j*GRIDSIZE,height);
      	}
    }

	/** la methode qui implemente le design pattern "factory".
		Peut tre rutilise telle quelle dans de nombreuses ralisations.
	*/
    Object createObject(String name){
          try{
            Class c = Class.forName(name);
            Object o = c.newInstance();
            return(o);
          }
          catch(ClassNotFoundException e){
            System.out.println("Classe inconnue : "+ e.getMessage());
          }
          catch(InstantiationException e){
            System.out.println("instantiation erreur : " + e.getMessage());
          }
          catch(IllegalAccessException e){
            System.out.println("Classe inconnue : " + e.getMessage());
          }
         return(null);
        }
    public void createFigure(String name){
    	setMode(MODE_DEFINE);
        fig = (Figure) createObject("maison."+name);
    }

    public void clear(){

			figures.removeAllElements(); // on supprime tout
			repaint(); // on demande un reaffichage
    }

	public void save(PrintWriter fos) {
		setMode(MODE_SELECT);
 		for (int i=0; i<figures.size();i++){
       		((Figure) figures.elementAt(i)).save(fos);
       	}
	}

	public void parse(StreamTokenizer st)  {
    	setMode(MODE_SELECT);
		figures.removeAllElements();
		try {
       		st.nextToken();

       		while ((st.ttype == st.TT_WORD) || (st.ttype == (int) '\"')){
       		    System.out.println(">> analyse de la figure " + st.sval);
				Figure o = (Figure) createObject(st.sval);
				o.parse(st,this);
				figures.addElement(o);
				st.nextToken();
			}
       }
       catch (IOException e) {
            	System.err.println("parse structure: " + e.getMessage());
       }
   }   // fin parse



	// ceux qui ne servent  rien mais qui sont
	// dfinis dans MouseListener.
	// on les implmente pour qu'ils ne fassent rien!
	public void mouseClicked(MouseEvent e){}
	public void mouseEntered(MouseEvent e){}
	public void mouseExited(MouseEvent e) {}
 	public void mouseMoved(MouseEvent e) {}

}
